
function [f,p,msg] = dlqrd(beta,A11,A12,A13,A22,A23,B1,Q,R,W,C1,C2,C3);
%DLQRD   Linear quadratic regulator design for discrete-time systems
%        with side constraints.
%        [f,p] = dlqrd(beta,A11,A12,A13,A22,A23,B1,Q,R,W,C1,C2,C3)
%	 calculates the optimal feedback gain matrix F such that
%        the feedback law:
%
%        (*)    u = -Fx = -F1*x1 - F2*x2
%
%        minimizes the cost function:
%
%		J = Sum beta^n{x[n]'Qx[n] + u[n]'Ru[n] + 2x[n]'Wu[n]}
%
%        subject to the constraint equations:
%
%	        x1[n+1] = A11*x1[n] + A12*x2[n] + A13*x3[n] + B1*u[n]
%	        x2[n+1] =             A22*x2[n] + A23*x3[n]
%
%        with x=[x1;x2;x3] and {x3[n]} given.  The solution (*)
%        is found by imposing the following side constraints (e.g.
%        market clearing conditions) on first-order conditions
%
%               x3[n] = C1*x1[n] + C2*x2[n] + C3*u[n].
%
%	 Also returned is P, the steady-state solution to the associated
%	 discrete matrix Riccati equation.
 
%	 Ellen McGrattan 1-22-89
%	 Revised 5-15-90, ERM
 
msg =1;
[n1,k]=size(B1);
[n2,n3]=size(A23);
n  = n1+n2;
B  = [B1;zeros(n2,k)];
A  = sqrt(beta)*([A11,A12,A13;zeros(n2,n1),A22,A23]-B/R*W');
A3 = A(:,n+1:n+n3);
A  = A(:,1:n);
W3 = W(n+1:n+n3,:);
B  = sqrt(beta)*B;
Q  = Q-W/R*W';
Q3 = Q(1:n,n+1:n+n3);
Q  = Q(1:n,1:n);
W  = W(1:n,:);
 
%    x3[t]=a1*[x1[t];x2[t]]+a2*mu[t+1]
c  = [C1,C2];
a1 = c-C3*((R+W3'*C3)\(W'+W3'*c));
a2 = -C3*((R+W3'*C3)\B');
 
I  = eye(n);
z  = zeros(n);
m1 = [I,B/R*B'-A3*a2;z,A'+Q3*a2];
m2 = [A+A3*a1,z;-Q-Q3*a1,I];
[v,d]=eig(m2,m1);
d = diag(d);
i = find(real(d)==NaN);
d(i) = inf*ones(length(i),1);
[d,index] = sort(abs(d));        % sort on magnitude of eigenvalues
if (~((d(n) < 1) & (d(n+1) > 1)))
  %error('Can''t order eigenvalues, (A,B) may be uncontrollable.')
  msg = 0; f=0; p=0;
  return
end
% select vectors with eigenvalues inside unit circle
chi = v(1:n,index(1:n));
lambda = v((n+1):(2*n),index(1:n));
p = real(lambda/chi);
r = R+B'*p*B;
f = (r*(eye(k)+R\W3'*C3)+B'*p*A3*C3)\(B'*p*(A+A3*c)+r/R*(W'+W3'*c));

