% socsec2.m a script file (not a function).  Prepares TWO sets of
% A,B,Q,R matrices for a Hansen-Sargent economy, but does not
% solve it.  The two sets are triggered by two
% ud matrices, one pre-retirement, the other
% post-retirement.  For use with selo social security project.
%
%   SOLVEA solves the social planning problem, to maximize
%
%      -.5 SUM beta^t [(s(t)-b(t)).(s(t)-b(t)) + l(t)^2]
%
%   subject to
%
%      phic * c(t) + phig * g(t) + phii * i(t) = gamma * k(t-1) + d(t)
%                              g(t).g(t) = l(t)^2
%                                   k(t) = deltak * k(t-1) + thetak * i(t)
%                                   h(t) = deltah * h(t-1) + thetah * c(t)
%                                   s(t) = lambda * h(t-1) + pih * c(t)
%                                 z(t+1) = a22 * z(t) + c2 * w(t+1)
%                                   b(t) = ub * z(t), d(t) = ud * z(t)
%   Where x(t) = [h(t-1)',k(t-1)',z(t)']', the solution of the problem is
%
%        x(t+1) = ao * x(t) + c * w(t+1)
%        j(t) = sj * x(t)
%   where j = k, h, c, s, g, i, b, and d.  The program also computes
%   Lagrange multipliers
%
%        muj = mj * x(t)
%   for j=k,c,h,s, and i.

%STEP 1:
%Computation of the dimension of the structural parameter matrices that the
%user has specified.  The user is told the dimension of the various
%matrices in order to help the user verify that the matrices are inputted
%correctly.

[nb,nh]=size(lambda);
[nd,nc]=size(phic);
[nz,nw]=size(c2);
[junk,ng]=size(phig);
[nk,ni]=size(thetak);


%STEP 2:
%Creation of the composite state vector x(t)'=[h(t-1)',k(t-1)',z(t)'] and its
%law of motion:  x(t+1) = A*x(t) + B*i(t) + C*w(t+1).

  phiin = inv([phic phig]);
  phiinc = [eye(nc) zeros(nc,ng)]*phiin;
  phiing = [zeros(ng,nc) eye(ng)]*phiin;
  b11 = - thetah*phiinc*phii;
  a1 = thetah*phiinc*gamma;
  a12 = [thetah*phiinc*ud1; zeros(nk,nz)];

    %Creation of the A matrix

     a11=[deltah a1 ; zeros(nk,nh) deltak];
     a=[a11 a12; [zeros(nz,nk+nh) a22]];

    %Creation of the B matrix

     b1=[b11; thetak];
     b=[b1; zeros(nz,ni)];

    %Creation of the C matrix

     c = [zeros(nk+nh,nw);c2];



%STEP 3:
%Creation of the matrices Q and R for the objective function of the optimal
%linear regulator problem.  The matrices Q, R, S are first created where
%the matrix S allows for the presence of cross products between the control
%and the state in the objective function.  The control is then transformed in
%the last step to remove the cross product and to put the problem in the proper
%form of the optimal linear regulator problem.


%STEP 3(i):
%Form matrices Q, R, S where S contains cross-products

   n=ni+nh+nk+nz;
   nx = nh+nk+nz;

   g1 = phiing*[-phii zeros(nd,nh) gamma ud1];
   g2 = [-pih*phiinc*phii lambda pih*phiinc*gamma -ub+pih*phiinc*ud1];

   r=g1'*g1 + g2'*g2;

   q=r(1:ni,1:ni);

   s=r(1:ni,1+ni:n);

   r=r(1+ni:n,1+ni:n);


%STEP 3(ii)
%Transform control to remove cross-product

   %First check if q is nonsingular

if rank(q) < max(size(q));

   error('EXECUTION STOPPED.  The 1-1 block of the R matrix is singular')

   end

   qi=inv(q);

   a=a-b*qi*s;

   r=r-s'*qi*s;

   a11=a(1:nh+nk,1:nh+nk);

   r1=r(1:nh+nk,1:nh+nk);


 
A1=a;
B1=b;
C1=c;
R1=r;
Q1=q;
Q1i=qi;
S1=s;




%  Do  the calculations for the ud2.


   %Creation of the A matrix

    clear a a11

     a11=[deltah a1 ; zeros(nk,nh) deltak];
     a=[a11 a12; [zeros(nz,nk+nh) a22]];

    %Creation of the B matrix

     b1=[b11; thetak];
     b=[b1; zeros(nz,ni)];



%STEP 3(i):
%Form matrices Q, R, S where S contains cross-products

   n=ni+nh+nk+nz;
   nx = nh+nk+nz;

   g1 = phiing*[-phii zeros(nd,nh) gamma ud2];
   g2 = [-pih*phiinc*phii lambda pih*phiinc*gamma -ub+pih*phiinc*ud2];

   r=g1'*g1 + g2'*g2;

   q=r(1:ni,1:ni);

   s=r(1:ni,1+ni:n);

   r=r(1+ni:n,1+ni:n);


%STEP 3(ii)
%Transform control to remove cross-product

   %First check if q is nonsingular

if rank(q) < max(size(q));

   error('EXECUTION STOPPED.  The 1-1 block of the R matrix is singular')

   end

   qi=inv(q);

   a=a-b*qi*s;

   r=r-s'*qi*s;

   a11=a(1:nh+nk,1:nh+nk);

   r1=r(1:nh+nk,1:nh+nk);


 
A2=a;
B2=b;
C2=c;
R2=r;
Q2=q;
Q2i=qi;
S2=s;




