%SOLVEHET.M a script file (not a function)
%SOLVEHET solves the pareto problem for two agents with heterogeneous
% household production functions, i.e. to maximize
%
% -.5{alpha ( SUM [(s1(t)-b1(t)).(s1(t)-b1(t)) + l1(t)^2] )
%         + (1-alpha) ( SUM [(s2(t)-b2(t)).(s2(t)-b2(t)) + l2(t)^2] ) }
%
%     subject to
%
%     phic * c(t) + phig * g(t) + phii * i(t) = gamma * k(t-1) + d(t)
%
%     gi(t).gi(t) = li(t)^2     i=1,2
%     g1(t) + g2(t) = g(t)
%
%     k(t) = deltak * k(t-1) + thetak * i(t)
%								
%     hi(t) = deltahi * hi(t-1) + thetahi * ci(t)
%     si(t) = lambdai * hi(t-1) + pihi * ci(t)  i=1,2
%
%   c1(t) + c2(t) = c(t)
%
%   z(t+1) = a22 * z(t) + c2 * w(t+1)
%   i(t) = ubi * z(t), i=1,2; d(t) = ud * z(t)
%
%   The state vector is x(t) = [h1(t-1)',h2(t-1)',k(t-1)',z(t)]'.
%   The control vector is u(t) = [c1(t)',i(t)'].
%   The solution of the problem is given by:
%
%        x(t+1) = ao * x(t) + c * w(t+1)
%        j(t) = sj * x(t)
%   for j = k,c,g,i,d, and ci,bi,gi,hi,si, for i=1,2.  The program also
%   computes lagrange multipliers.
%
%   NOTE WELL: for models with durable consumption, the parameterization
%   must include current consumption goods in current services.
%
%
% SEE THE COMMENTS FOR THE VALUATION SECTION OF THE PROGRAM AT THE END
%


%SEARCH ROUTINE

bal1 = 5;
i = 1;
       alpha = .5;
tol = 1e-5;

while abs(bal1) > tol;



% STEP 1:
% Computation of the dimensions of the structural parameter matrices 
% specified by the user.  NOTE:  the household technology matrices must be of
% the same dimension for each consumer.

[nb,nh]=size(lambda1);
[nd,nc]=size(phic);
[nz,nw]=size(c2);
[junk,ng]=size(phig);
[nk,ni]=size(thetak);


% STEP 2:
% Creation of the composite state vector x(t)'=[h1(t-1)',h2(t-1)',k(t-1)',z(t)']
% and its law of motion:  x(t+1) = A*x(t) + B*u(t) + C*w(t+1).

	phiin = inv([phic phig]);
	phiinc = [eye(nc) zeros(nc,ng)]*phiin;
	phiing = [zeros(ng,nc) eye(ng)]*phiin;
	a1 = [deltah1 zeros(nh,nh+nk)];
	a2 = [zeros(nh,nh) deltah2 thetah2*phiinc*gamma];
	a3 = [zeros(nk,nh+nh) deltak];
	a4 = thetah2*phiinc*ud;
	b5 = -thetah2*phiinc*phii;
	b6 = [thetah1 zeros(nh,ni)];
	b7 = [-thetah2 b5];
	b8 = [zeros(nk,nc) thetak];

%Creation of the A matrix:

	a11 = [a1 ; a2 ; a3];
	a12 = [zeros(nh,nz) ; a4 ; zeros(nk,nz)];
	a = [a11 a12 ; [zeros(nz,nh+nh+nk) a22]];

%Creation of the B matrix:

	b1 = [b6 ; b7 ; b8];
	b = [b1 ; zeros(nz,nc+ni)];

%Creation of the C matrix:

	c = [zeros(nh+nh+nk,nw); c2];

%Scaling of the law of motion by the square root of the discount factor to
%account for the discounting in the optimization problem:

	a = a*sqrt(beta);
	a11 = a11*sqrt(beta);
	b1 = b1*sqrt(beta);
	b = b*sqrt(beta);


% STEP 3:
% Creation of the matrices Q and R for the objective function of the optimal
% linear regulator problem.  The matrices Q, R, S are first created where S
% allows for the presence of cross products between the control and the state 
% in the objective function.  The control is then transformed in the last step
% to remove the cross product and to put the problem in the proper form of the
% optimal linear regulator problem.

% STEP 3a:
% Form matrices Q, R, S where S contains cross products:

	nu = nc+ni;
	nx = nh+nh+nk+nz;
	n = nu+nx;

	h1 = [pih1 zeros(nb,ni) lambda1 zeros(nb,nh+nk) -ub1];
	h21 = [-pih2 -pih2*phiinc*phii];
	h22 = [zeros(nb,nh) lambda2 pih2*phiinc*gamma -ub2+pih2*phiinc*ud];
	h2 = [h21 h22];

	g1 = [zeros(ng,nc) -phiing*phii zeros(ng,nh+nh) phiing*gamma phiing*ud];

	r = alpha*h1'*h1 + (1-alpha)*(h2'*h2) + alpha*(1-alpha)*(g1'*g1);

	q = r(1:nu,1:nu);

	s = r(1:nu,nu+1:n);

	r = r(nu+1:n,nu+1:n);

% STEP 3b:
% Transform control to remove cross product:

% First check if q is non-singular:

if rank(q) < max(size(q));

	error('EXECUTION STOPPED.  The (1,1) block of the R matrix is singular')
	
	end
	
	qi = inv(q);

	a = a - b*qi*s;

	r = r - s'*qi*s;

	a11 = a(1:nh+nh+nk,1:nh+nh+nk);

	r1 = r(1:nh+nh+nk,1:nh+nh+nk);


% STEP 4:
% Solve the Ricatti equation using two doubling algorithms.
% We exploit the fact that the solution to the Riccatti equation can be solved
% in parts (endogenous and exogenous).

	disp('Calculating, please wait')

% STEP 4a:
% Calculate the (1,1) block of the Riccatti matrix.  The result is a matrix kg
% such that kg'*[h1(t), h2(t), k(t)] is the feedback part of the control law.
% The matrix a11o gives the (1,1) (all endogenous variables) part of ao.

	[kg,v11] = doubleo(a11',b1',r1,q);

	a11o = a11 - b1*kg';

% STEP 4b:
% Calculate the (1,2) block of the Riccatti matrix.  The result is the matrix
% kg2 such that kg2*z(t) is the feedforward part of the control law.

	a22t = a(nh+nh+nk+1:nx,nh+nh+nk+1:nx);

	r2 = r(1:nh+nh+nk,nh+nh+nk+1:nx);

	a12 = a(1:nh+nh+nk,nh+nh+nk+1:nx);

	qopt = inv(q + b1'*v11*b1);

	r12 = r2 - a11'*v11*b1*qopt*b1'*v11*a12 + a11'*v11*a12;

	v12 = doublej2(a11o',r12,a22t',eye(nz));

	kg2 = qopt*b1'*(v12*a22t + v11*a12);

% STEP 4c:
% Compute the matrix ao which gives the reduced form law of motion for x(t)
% of the form:  x(t+1) = ao*x(t) + c*w(t+1).  The matrix ao also has the
% square root of beta rescaling unwound.

	ao = (a - b*[kg' kg2])/sqrt(beta);    %square root of beta unwound.

	a11o = a11o/sqrt(beta);  %square root of beta unwound in endogenous state
                            %variable part of ao.

% STEP 5:
% Compute endogenous and exogenous eigenvalues.

% Endogenous eigenvalues:

	endo = eig(a11o);

% Exogenous eigenvalues:

	exo = eig(a22);


% STEP 6:
% Calculation of the solution matrices.

	sh1 = ao(1:nh,1:nx);
	sh2 = ao(nh+1:nh+nh,1:nx);
	sk = ao(nh+nh+1:nh+nh+nk,1:nx);
	skl = [zeros(nk,nh+nh) eye(nk) zeros(nk,nz)];
	sd = [zeros(nd,nh+nh+nk) ud];
	sb1 = [zeros(nb,nh+nh+nk) ub1];
	sb2 = [zeros(nb,nh+nh+nk) ub2];
	su = -[kg' kg2] - qi*s;
	sc1 = su(1:nc,1:nx);
	si = su(nc+1:nu,1:nx);
	sc = phiinc*(gamma*skl + sd - phii*si);
	sg = phiing*(gamma*skl + sd - phii*si);
	sc2 = sc - sc1;
	sg1 = (1-alpha)*sg;
	sg2 = sg - sg1;
	ss1 = lambda1*[eye(nh) zeros(nh,nh+nk+nz)] + pih1*sc1;
	ss2 = lambda2*[zeros(nh,nh) eye(nh)	zeros(nh,nk+nz)] + pih2*sc2;

%disp('The matrix ao has been calculated for the law of motion for the')
%disp('entire state vector.  This matrix satisfies:')
%disp('  ')
%disp('       x(t+1) = ao*x(t) + c*w(t+1).  ')
%disp('  ')
%disp('The endogenous eigenvalues are in the vector endo and the ')
%disp('exogenous eigenvalues are in the vector exo.')
%disp('  ')
%disp('  ')

%disp('The solution to the model is given by j(t) = sj*x(t), for ')
%disp('j = k,c,g,i,d and ci,bi,gi,hi,si for i=1,2.')
%disp('All of the above sj matrices have been computed and can be used')
%disp('in other MATLAB programs.')

% STEP 7 
% Calculation of the Lagrange multipliers.

	mh1 = - beta * [eye(nh) zeros(nh,nh+nk)] * [v11 v12] * ao;
	mh2 = - beta * [zeros(nh,nh) eye(nh) zeros(nh,nk)] * [v11 v12] * ao;
	mk = - beta * [zeros(nk,nh+nh) eye(nk)] * [v11 v12] * ao;
	ms1 = alpha * (sb1 -ss1);
	ms2 = (1 - alpha) * (sb2 - ss2);
	mc1 = thetah1'*mh1 + pih1'*ms1;
	mc2 = thetah2'*mh2 + pih2'*ms2;
	mc = mc1;
	mi = thetak'*mk;
	mg = (1 - alpha) * sg2;
	md = phiin'*[mc ; -mg];
	mg1 = mg;
	mg2 = mg;

%disp('   ')
%disp('   ')

%disp('The shadow prices satisfy Mj(t) = mj*x(t) for j as above.')
%disp('They can be used in other MATLAB programs.')
%disp('   ')
%disp('   ')

%disp('Your equilibrium has been calculated and you are now ready to play')
%disp('with the economy.')
 
	

%VALUES.M a script file (not a function)
%	VALUES computes the values of the consumers' expenditure and wealth
%	for the sake of budget constraint evaluation, i.e.
%
%		E Sum {beta^t*p0(t).ci(t)}| I0 = E Sum {beta^t*(w0(t).li(t)
%                                           + alph0(t).di(t)0}| I0 +v0ki(-1)
%
%	The program SOLVEHET.M must be run prior to running this program.
%	The values of udi, k0i, and x0 must also be in memory.

% Compute the generalized variance:

	v = doublej2(beta*ao,c,ao,c);
	v = v*beta/(1 - beta);

% Compute the value of the technology shock:

	sd1 = [zeros(nd,nh+nh+nk) ud1]; 
	sd2 = [zeros(nd,nh+nh+nk) ud2]; 

	wd1 = doublej2(beta*ao',md',ao',sd1');
	wd1 = x0'*wd1*x0 + sum(diag(md*v*sd1'));

	wd2 = doublej2(beta*ao',md',ao',sd2');
	wd2 = x0'*wd2*x0 + sum(diag(md*v*sd2'));

% Compute the value of the household input:

	wg1 = doublej2(beta*ao',mg',ao',sg1');
	wg1 = x0'*wg1*x0 + sum(diag(mg*v*sg1'));

	wg2 = doublej2(beta*ao',mg',ao',sg2');
	wg2 = x0'*wg2*x0 + sum(diag(mg*v*sg2'));

% Compute the value of physical capital:

	wk1 = k01'*(deltak'*mk + gamma'*md)*x0;
	wk2 = k02'*(deltak'*mk + gamma'*md)*x0;

% Compute the value of consumption goods:

	wc1 = doublej2(beta*ao',mc',ao',sc1');
	wc1 = x0'*wc1*x0 + sum(diag(mc*v*sc1'));

	wc2 = doublej2(beta*ao',mc',ao',sc2');
	wc2 = x0'*wc2*x0 + sum(diag(mc*v*sc2'));

% Compute expenditure and wealth:

	spend1 = wc1;
	spend2 = wc2;

	wealth1 = wd1 + wg1 + wk1;
	wealth2 = wd2 + wg2 + wk2;

% Compute budget balance:

	bal1 = wealth1 - spend1
	bal2 = wealth2 - spend2;

%disp('    ')
%disp('The values of consumer wealth, expenditure, and budget balance')
%disp('have been calculated.')
%disp('They can be found in spendi, wealthi, and bali, for i=1,2.')


i = i + 1
 
       if bal1 > 0
              alpha = alpha + 2^(-i);

       else
              alpha = alpha - 2^(-i);

    end;

alpha

end;

	 					
